
#########################################################################
#  The MIT License (MIT)
#  
#  Copyright (c) 2013 CIVA LIN (林雪凡)
#  The Anobii Fetcher v-1.0 is licensed under the MIT license provided
#  below by supporting from the sponsor "Open Source Software Foundry, OSSF"
#  of Academia Sinica at: http://www.openfoundry.org/
#  
#  Permission is hereby granted, free of charge, to any person obtaining a
#  copy of this software and associated documentation files
#  (the "Software"), to deal in the Software without restriction, including
#  without limitation the rights to use, copy, modify, merge, publish,
#  distribute, sublicense, and/or sell copies of the Software, and to
#  permit persons to whom the Software is furnished to do so,
#  subject to the following conditions:
#  
#  The above copyright notice and this permission notice shall be included
#  in all copies or substantial portions of the Software.
#  
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
#  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
#  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
#  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
#  CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
#  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#  SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
##########################################################################

from urllib import quote

from calibre.ebooks.metadata.sources.base import Source
from calibre.ebooks.metadata.book.base import Metadata

def percentilize(word):
    '''
    change to percentage encoding
    '''
    return quote(
            word.encode('utf-8') if isinstance(word, unicode) else word)

class AnobiiMeta(Source):

    name        = 'Anobii Fetcher'
    description = 'Get the metadata of book from anobii.com'
    supported_platforms = ['windows', 'osx', 'linux']
    author      = u'Civa Lin'
    version     = (1, 0, 0)
    minimum_calibre_version = (0, 8, 60)

    capabilities = frozenset(['identify', 'cover'])
    touched_fields = frozenset([ 'title', 'authors',
        'pubdate', 'comments', 'publisher',
        'identifier:isbn', 'identifier:anobii_id'])
    has_html_comments = True
    cached_cover_url_is_reliable = True
    supports_gzip_transfer_encoding = True

    idname = 'anobii_id' # site used id name, please add to touched_field

    # ======== No Need to Change =============

    def identify(self, log, result_queue, abort,
            title = None, authors = None, identifiers = {}, timeout = 30):

        querys = self.create_querys(title = title, authors = authors,
                identifiers = identifiers)

        # analysis raw content
        urls = self.get_book_urls(querys = querys)

        import threading
        threads = []
        for url in urls:
            thread = threading.Thread(target = self.fetch_item_metadata,
                    kwargs = {'url': url, 'result_queue': result_queue})
            thread.start()
            threads.append(thread)
        for t in threads:
            t.join()

    def fetch_item_metadata(self, url, result_queue):
        '''analysis individual book '''
        from lxml import html

        root = html.parse(url)

        # title and authors
        mi = Metadata(title = self.get_title(root),
                authors = self.get_authors(root))
        mi.identifiers = {}

        # site_id
        mi.identifiers[type(self).idname] = self.get_siteid(root)

        # isbn
        mi.isbn = self.get_isbn(root)

        # publisher
        mi.publisher = self.get_publisher(root)

        # pubdate
        mi.pubdate = self.get_pubdate(root)

        # comment / description
        mi.comments = self.get_desc(root)

        # add to queue
        mi.source_relevance = 0
        self.clean_downloaded_metadata(mi)
        result_queue.put(mi)

    def download_cover(self, log, result_queue, abort, title = None, authors = None, identifiers = {}, timeout = 30, get_best_cover = False):

        cached_url = self.get_cached_cover_url(identifiers = identifiers)
        if abort.is_set():
            return

        br = self.browser
        try:
            cdata = br.open_novisit(cached_url, timeout = timeout).read()
            if cdata:
                result_queue.put( (self, cdata) )
        except:
            log.exception('Failed to download cover from:', cached_url)

    # ======== Get Search Result =============

    def create_querys(self, title = None, authors = None, identifiers = {}):
        ''' create multiple query url of index-of-content '''
        SEARCH_URL = u'http://www.anobii.com/search?keyword=%s'

        keywords = [title or '']
        keywords.extend(authors or [])
        keywords = [ percentilize(word) for word in keywords ]

        query = SEARCH_URL % '+'.join(keywords)

        return [ query ]

    def get_book_urls(self, querys):
        '''analysis index-of-content'''
        from lxml import html
        SITE_URL = 'http://anobii.com'

        roots = [ html.parse(query) for query in querys ]

        urls = []

        for root in roots:
            books = root.findall('//div[@class="shelf"]//tr')
            urls.extend([ SITE_URL + percentilize(e.find('.//a[@class="cover_image"]').get('href')) for e in books])

        return urls

    # ======== Cover Download ================

    def get_cached_cover_url(self, identifiers):
        IMG = 'http://image.anobii.com/anobi/image_book.php?item_id=%s'
        return IMG % identifiers.get('anobii_id')

    # ======== Access Book Info Page =========

    def get_book_url(self, identifiers):
        URL = 'http://www.anobii.com/books/%s/'

        aid = identifiers.get('anobii_id', None)
        if aid is not None:
            return ('anobii_id', aid, URL % aid)

    # ======== Get Book Metadata =============

    def get_title(self, root):
        ''' use root element to get something '''
        title = root.findtext('//h1[@class="title"]')
        return title

    def get_authors(self, root):
        ''' use root element to get something '''
        authors = [e.text
                for e in root.findall('//p[@class="contributor"]/a')]
        return authors

    def get_siteid(self, root):
        ''' use root element to get something '''
        siteid = root.xpath('//input[@id="encryptEditionId"]')[0].get('value')
        return siteid

    def get_isbn(self, root):
        ''' use root element to get something '''
        isbn13 = root.xpath('//ul[@class="details"]//span[.="ISBN-13:"]/following-sibling::strong/text()')
        if isbn13:
            isbn = isbn13[0]
        else:
            isbn10 = root.xpath('//ul[@class="details"]//span[.="ISBN-10:"]/following-sibling::strong/text()')
            if isbn10:
                isbn = isbn10[0]
            else:
                isbn = None
        return isbn

    def get_publisher(self, root):
        ''' use root element to get something '''
        publishers = root.xpath('//ul[@class="details"]//span[.="Publisher:"]/following-sibling::strong/text()')
        if publishers:
            publisher = publishers[0]
        else:
            publisher = None
        return publisher

    def get_pubdate(self, root):
        ''' use root element to get something '''
        from calibre.utils.date import parse_date
        pubdates = root.xpath('//ul[@class="details"]//span[.="Publish date:"]/following-sibling::strong/text()')
        if pubdates:
            pubdate = parse_date(pubdates[0], assume_utc = True)
        else:
            pubdate = None
        return pubdate

    def get_desc(self, root):
        ''' use root element to get something '''
        from lxml import etree
        e_desc = root.find('//div[@id="description_full"]')
        if e_desc.text_content().strip(' \n\t'):
            desc = etree.tostring(e_desc)
        else:
            desc = None
        return desc
